// {{{RME classifier 'Logical View::VANT::VCTRL::VPOTE::EprControl'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "EprControl.h"
#endif

#include <RTSystem/VANT.h>
#include <EprControl.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_controller"
	  , &Epr_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_EprControl_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new EprControl_Actor( _rts, _ref );
}

const RTActorClass EprControl =
{
	(const RTActorClass *)0
  , "EprControl"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_EprControl_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Active"
};

#define SUPER RTActor

EprControl_Actor::EprControl_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, epr_status( 1 )
{
}

EprControl_Actor::~EprControl_Actor( void )
{
}

int EprControl_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_controller
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_controller;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Active:J4359EA40009E:request_epr'
INLINE_METHODS void EprControl_Actor::transition2_request_epr( const void * rtdata, Epr_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_controller.epr_status(epr_status).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359EA9E0179:up'
INLINE_METHODS void EprControl_Actor::transition3_up( const void * rtdata, Epr_Protocol::Conjugate * rtport )
{
	// {{{USR
	epr_status=2;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359EAC200BD:down'
INLINE_METHODS void EprControl_Actor::transition4_down( const void * rtdata, Epr_Protocol::Conjugate * rtport )
{
	// {{{USR
	epr_status=0;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359EADC034B:straight'
INLINE_METHODS void EprControl_Actor::transition5_straight( const void * rtdata, Epr_Protocol::Conjugate * rtport )
{
	// {{{USR
	epr_status=1;
	// }}}USR
}
// }}}RME

INLINE_CHAINS void EprControl_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void EprControl_Actor::chain3_up( void )
{
	// transition ':TOP:Active:J4359EA9E0179:up'
	rtgChainBegin( 2, "up" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_up( msg->data, (Epr_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void EprControl_Actor::chain4_down( void )
{
	// transition ':TOP:Active:J4359EAC200BD:down'
	rtgChainBegin( 2, "down" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_down( msg->data, (Epr_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void EprControl_Actor::chain2_request_epr( void )
{
	// transition ':TOP:Active:J4359EA40009E:request_epr'
	rtgChainBegin( 2, "request_epr" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_request_epr( msg->data, (Epr_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void EprControl_Actor::chain5_straight( void )
{
	// transition ':TOP:Active:J4359EADC034B:straight'
	rtgChainBegin( 2, "straight" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_straight( msg->data, (Epr_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void EprControl_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Active'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_controller'
				switch( signalIndex )
				{
				case Epr_Protocol::Conjugate::rti_do_epr_up:
					chain3_up();
					return;
				case Epr_Protocol::Conjugate::rti_do_epr_down:
					chain4_down();
					return;
				case Epr_Protocol::Conjugate::rti_request_epr_status:
					chain2_request_epr();
					return;
				case Epr_Protocol::Conjugate::rti_do_straight:
					chain5_straight();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * EprControl_Actor::getActorData( void ) const
{
	return &EprControl_Actor::rtg_class;
}

const RTActor_class EprControl_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 2
  , EprControl_Actor::rtg_parent_state
  , &EprControl
  , 0
  , (const RTComponentDescriptor *)0
  , 1
  , EprControl_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 1
  , EprControl_Actor::rtg_EprControl_fields
};

const RTStateId EprControl_Actor::rtg_parent_state[] =
{
	0
  , 1
};

const RTPortDescriptor EprControl_Actor::rtg_ports[] =
{
	{
		"to_controller"
	  , (const char *)0
	  , &Epr_Protocol::Conjugate::rt_class
	  , RTOffsetOf( EprControl_Actor, EprControl_Actor::to_controller )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

const RTFieldDescriptor EprControl_Actor::rtg_EprControl_fields[] =
{
	// {{{RME classAttribute 'epr_status'
	{
		"epr_status"
	  , RTOffsetOf( EprControl_Actor, epr_status )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
