// {{{RME classifier 'Logical View::VANT::VCTRL::VCONT::FlightControl'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "FlightControl.h"
#endif

#include <RTSystem/VANT.h>
#include <FlightControl.h>
#include <MissionVant.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_bus"
	  , &Bus_Cont_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
  , {
		"to_power"
	  , &Power_Cont_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
  , {
		"to_aileron"
	  , &Aileron_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_elevator"
	  , &Elevator_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_flap"
	  , &Flap_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_rudder"
	  , &Rudder_Protocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_FlightControl_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new FlightControl_Actor( _rts, _ref );
}

const RTActorClass FlightControl =
{
	(const RTActorClass *)0
  , "FlightControl"
  , (RTVersionId)0
  , 6
  , rtg_relays
  , new_FlightControl_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "IDLE"
  , "START"
  , "TAKING_OFF"
  , "NORMAL_FLIGHT"
  , "LANDING"
  , "receiveMission"
};

#define SUPER RTActor

// {{{RME classAttribute 'myMission'
MissionVant * FlightControl_Actor::myMission;
// }}}RME

FlightControl_Actor::FlightControl_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

FlightControl_Actor::~FlightControl_Actor( void )
{
}

int FlightControl_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_bus
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_bus;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// to_power
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_power;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 2:
		// to_aileron
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_aileron;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 3:
		// to_elevator
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_elevator;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 4:
		// to_flap
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_flap;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 5:
		// to_rudder
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_rudder;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME enter ':TOP:IDLE'
INLINE_METHODS void FlightControl_Actor::enter2_IDLE( void )
{
	// {{{USR
	to_flap.do_flap_straight().send();
	to_elevator.do_straight().send();
	to_aileron.do_aileron_straight().send();
	to_rudder.do_rudder_straight().send();
	to_power.do_engine_off().send();
	// }}}USR
}
// }}}RME

void FlightControl_Actor::enterStateV( void )
{
	switch( getCurrentState() )
	{
	case 2:
		enter2_IDLE();
		break;
	default:
		RTActor::enterStateV();
		break;
	}
}

// {{{RME transition ':TOP:IDLE:J4359EDC60139:to_start'
INLINE_METHODS void FlightControl_Actor::transition2_to_start( const void * rtdata, Bus_Cont_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_power.do_engine_on().send();
	to_power.request_full_power().send();
	timer1.informIn(RTTimespec(10,0));

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:START:J4359F592007E:to_taking_off'
INLINE_METHODS void FlightControl_Actor::transition3_to_taking_off( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	to_elevator.do_elevator_up().send();
	timer1.informIn(RTTimespec(50,0));

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:TAKING_OFF:J4359F64A02C0:to_normal'
INLINE_METHODS void FlightControl_Actor::transition4_to_normal( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	to_elevator.do_straight().send();
	to_rudder.do_rudder_straight().send();
	to_flap.do_flap_straight().send();
	to_aileron.do_aileron_straight().send();
	to_power.decrease_power().send();
	timer1.informIn(RTTimespec(180,0));
	timer2.informIn(RTTimespec(20,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:NORMAL_FLIGHT:J4359F7890224:timer_pos'
INLINE_METHODS void FlightControl_Actor::transition5_timer_pos( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	to_bus.request_position().send();
	to_bus.request_destination().send();
	timer2.informIn(RTTimespec(20,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:NORMAL_FLIGHT:J4359F81E00EC:position'
INLINE_METHODS void FlightControl_Actor::transition6_position( const int * rtdata, Bus_Cont_Protocol::Conjugate * rtport )
{
	// {{{USR
	position = (int)*rtdata;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:NORMAL_FLIGHT:J4359F8B80040:destination'
INLINE_METHODS void FlightControl_Actor::transition7_destination( const int * rtdata, Bus_Cont_Protocol::Conjugate * rtport )
{
	// {{{USR
	destination = (int)*rtdata;
	if ((destination > 0) && (position < 0))
	{
		to_rudder.do_rudder_right().send();
		to_aileron.do_aileron_right().send();
	}
	else if ((destination > 0) && (position < 0))
	{
		to_rudder.do_rudder_left().send();
		to_aileron.do_aileron_left().send();	
	}
	else if ((destination > 0) && (position > 0))
	{
		to_rudder.do_rudder_straight().send();
		to_aileron.do_aileron_straight().send();	
	}
	else if ((destination < 0) && (position < 0))
	{
		to_rudder.do_rudder_straight().send();
		to_aileron.do_aileron_straight().send();	
	}
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:NORMAL_FLIGHT:J4359FA2F02CE:to_landing'
INLINE_METHODS void FlightControl_Actor::transition8_to_landing( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	to_power.request_low_power().send();
	to_elevator.do_elevator_down().send();
	to_flap.do_flap_down().send();
	to_rudder.do_rudder_straight().send();
	to_aileron.do_aileron_straight().send();
	timer1.informIn(RTTimespec(60,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:IDLE:J437F843502E0:receiveMission'
INLINE_METHODS void FlightControl_Actor::transition10_receiveMission( const MissionVant * rtdata, Bus_Cont_Protocol::Conjugate * rtport )
{
	// {{{USR
	printf("\nMSG_CTRL_FlightControl: Message was received from VCNS-Navigation\n");

	myMission = ((MissionVant*) getMsg()->getData()); 

	// printar os valores
	printf("MSG_CTRL_FlightControl:Mission Altidude: %d\n",myMission->Altitude); 
	printf("MSG_CTRL_FlightControl:Mission Latitude: %d\n",myMission->Latitude);  
	printf("MSG_CTRL_FlightControl:Mission Longitude: %d\n",myMission->Longitude);
	printf("MSG_CTRL_FlightControl:Mission Atack?");
	if (myMission->Missao)
		printf("Yes ATACK!\n");
	else
		printf("No JUST IMAGE!\n");
	printf("MSG_CTRL_FlightControl:Mission Image: waiting...\n\n");

	printf("MSG_CTRL_FlightControl:Atack/Image in 2 seconds...\n\n");

	// timer
	timer1.informIn(RTTimespec(2,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:receiveMission:J438740C4029F:timeout'
INLINE_METHODS void FlightControl_Actor::transition11_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	// chegou no ponto de fazer o ataque/foto
	printf("\n\nMSG_CTRL_FlightControl: DO ACTION NOW, WE WAS AT CORRECT POINT!!!\n");
	to_bus.doAction(*myMission).send();

	// }}}USR
}
// }}}RME

INLINE_CHAINS void FlightControl_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FlightControl_Actor::chain10_receiveMission( void )
{
	// transition ':TOP:IDLE:J437F843502E0:receiveMission'
	rtgChainBegin( 2, "receiveMission" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition10_receiveMission( (const MissionVant *)msg->data, (Bus_Cont_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 7 );
}

INLINE_CHAINS void FlightControl_Actor::chain2_to_start( void )
{
	// transition ':TOP:IDLE:J4359EDC60139:to_start'
	rtgChainBegin( 2, "to_start" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_to_start( msg->data, (Bus_Cont_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void FlightControl_Actor::chain3_to_taking_off( void )
{
	// transition ':TOP:START:J4359F592007E:to_taking_off'
	rtgChainBegin( 3, "to_taking_off" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_to_taking_off( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void FlightControl_Actor::chain4_to_normal( void )
{
	// transition ':TOP:TAKING_OFF:J4359F64A02C0:to_normal'
	rtgChainBegin( 4, "to_normal" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_to_normal( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void FlightControl_Actor::chain6_position( void )
{
	// transition ':TOP:NORMAL_FLIGHT:J4359F81E00EC:position'
	rtgChainBegin( 5, "position" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_position( (const int *)msg->data, (Bus_Cont_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void FlightControl_Actor::chain7_destination( void )
{
	// transition ':TOP:NORMAL_FLIGHT:J4359F8B80040:destination'
	rtgChainBegin( 5, "destination" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition7_destination( (const int *)msg->data, (Bus_Cont_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void FlightControl_Actor::chain8_to_landing( void )
{
	// transition ':TOP:NORMAL_FLIGHT:J4359FA2F02CE:to_landing'
	rtgChainBegin( 5, "to_landing" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition8_to_landing( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 6 );
}

INLINE_CHAINS void FlightControl_Actor::chain5_timer_pos( void )
{
	// transition ':TOP:NORMAL_FLIGHT:J4359F7890224:timer_pos'
	rtgChainBegin( 5, "timer_pos" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_timer_pos( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void FlightControl_Actor::chain9_t1( void )
{
	// transition ':TOP:LANDING:J4359FB380001:t1'
	rtgChainBegin( 6, "t1" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FlightControl_Actor::chain11_timeout( void )
{
	// transition ':TOP:receiveMission:J438740C4029F:timeout'
	rtgChainBegin( 7, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition11_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void FlightControl_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:IDLE'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_bus'
				switch( signalIndex )
				{
				case Bus_Cont_Protocol::Conjugate::rti_sendMission:
					chain10_receiveMission();
					return;
				case Bus_Cont_Protocol::Conjugate::rti_do_take_off:
					chain2_to_start();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:START'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 7:
				// {{{RME port 'timer1'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain3_to_taking_off();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:TAKING_OFF'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 7:
				// {{{RME port 'timer1'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain4_to_normal();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 5:
			// {{{RME state ':TOP:NORMAL_FLIGHT'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_bus'
				switch( signalIndex )
				{
				case Bus_Cont_Protocol::Conjugate::rti_position_value:
					chain6_position();
					return;
				case Bus_Cont_Protocol::Conjugate::rti_destination_value:
					chain7_destination();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 7:
				// {{{RME port 'timer1'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain8_to_landing();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 8:
				// {{{RME port 'timer2'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain5_timer_pos();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 6:
			// {{{RME state ':TOP:LANDING'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 7:
				// {{{RME port 'timer1'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain9_t1();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 7:
			// {{{RME state ':TOP:receiveMission'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 7:
				// {{{RME port 'timer1'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain11_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * FlightControl_Actor::getActorData( void ) const
{
	return &FlightControl_Actor::rtg_class;
}

const RTActor_class FlightControl_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 7
  , FlightControl_Actor::rtg_parent_state
  , &FlightControl
  , 0
  , (const RTComponentDescriptor *)0
  , 8
  , FlightControl_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 2
  , FlightControl_Actor::rtg_FlightControl_fields
};

const RTStateId FlightControl_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
  , 1
  , 1
  , 1
  , 1
};

const RTPortDescriptor FlightControl_Actor::rtg_ports[] =
{
	{
		"to_bus"
	  , (const char *)0
	  , &Bus_Cont_Protocol::Conjugate::rt_class
	  , RTOffsetOf( FlightControl_Actor, FlightControl_Actor::to_bus )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_power"
	  , (const char *)0
	  , &Power_Cont_Protocol::Conjugate::rt_class
	  , RTOffsetOf( FlightControl_Actor, FlightControl_Actor::to_power )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_aileron"
	  , (const char *)0
	  , &Aileron_Protocol::Base::rt_class
	  , RTOffsetOf( FlightControl_Actor, FlightControl_Actor::to_aileron )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_elevator"
	  , (const char *)0
	  , &Elevator_Protocol::Base::rt_class
	  , RTOffsetOf( FlightControl_Actor, FlightControl_Actor::to_elevator )
	  , 1 // cardinality
	  , 4
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_flap"
	  , (const char *)0
	  , &Flap_Protocol::Base::rt_class
	  , RTOffsetOf( FlightControl_Actor, FlightControl_Actor::to_flap )
	  , 1 // cardinality
	  , 5
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_rudder"
	  , (const char *)0
	  , &Rudder_Protocol::Base::rt_class
	  , RTOffsetOf( FlightControl_Actor, FlightControl_Actor::to_rudder )
	  , 1 // cardinality
	  , 6
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"timer1"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( FlightControl_Actor, FlightControl_Actor::timer1 )
	  , 1 // cardinality
	  , 7
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"timer2"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( FlightControl_Actor, FlightControl_Actor::timer2 )
	  , 1 // cardinality
	  , 8
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
};

const RTFieldDescriptor FlightControl_Actor::rtg_FlightControl_fields[] =
{
	// {{{RME classAttribute 'position'
	{
		"position"
	  , RTOffsetOf( FlightControl_Actor, position )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'destination'
  , {
		"destination"
	  , RTOffsetOf( FlightControl_Actor, destination )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
