// {{{RME classifier 'Logical View::VANT::VCTRL::VCOMB::FuelController'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "FuelController.h"
#endif

#include <RTSystem/VANT.h>
#include <FuelController.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_bus"
	  , &Bus_Comb_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
  , {
		"to_power"
	  , &Power_Comb_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
  , {
		"to_consume"
	  , &Consume_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_mix"
	  , &Mix_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_level"
	  , &Level_Protocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_FuelController_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new FuelController_Actor( _rts, _ref );
}

const RTActorClass FuelController =
{
	(const RTActorClass *)0
  , "FuelController"
  , (RTVersionId)0
  , 5
  , rtg_relays
  , new_FuelController_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Active"
  , "Process_Bus_Consume_Request"
  , "Process_Bus_Level_Request"
  , "Process_Bus_Mix_Request"
  , "Process_Power_Level_Request"
  , "Process_Power_Mix_Request"
  , "Process_Power_Consume_Request"
  , "Process_Unload"
};

#define SUPER RTActor

FuelController_Actor::FuelController_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

FuelController_Actor::~FuelController_Actor( void )
{
}

int FuelController_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_bus
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_bus;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// to_power
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_power;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 2:
		// to_consume
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_consume;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 3:
		// to_mix
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_mix;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 4:
		// to_level
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_level;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Active:J4359B2AE0214:request_consume'
INLINE_METHODS void FuelController_Actor::transition2_request_consume( const void * rtdata, Bus_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_consume.request_fuel_consume().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359B5000197:request_bus_level'
INLINE_METHODS void FuelController_Actor::transition3_request_bus_level( const void * rtdata, Bus_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_level.request_fuel_level().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Process_Bus_Consume_Request:J4359B6130158:consume_to_bus'
INLINE_METHODS void FuelController_Actor::transition4_consume_to_bus( const int * rtdata, Consume_Protocol::Base * rtport )
{
	// {{{USR
	to_bus.consume_value((int)*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Process_Bus_Level_Request:J4359B64E0223:level_to_bus'
INLINE_METHODS void FuelController_Actor::transition5_level_to_bus( const int * rtdata, Level_Protocol::Base * rtport )
{
	// {{{USR
	to_bus.level_value((int)*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359B780037B:request_mix'
INLINE_METHODS void FuelController_Actor::transition6_request_mix( const void * rtdata, Bus_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_mix.request_fuel_mix().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Process_Bus_Mix_Request:J4359B7D202DF:mix_to_bus'
INLINE_METHODS void FuelController_Actor::transition7_mix_to_bus( const bool * rtdata, Mix_Protocol::Base * rtport )
{
	// {{{USR
	to_bus.mix_value((int)*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359BA6001A6:request_consume_power'
INLINE_METHODS void FuelController_Actor::transition8_request_consume_power( const void * rtdata, Power_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_consume.request_fuel_consume().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359BA7D0158:request_mix_power'
INLINE_METHODS void FuelController_Actor::transition9_request_mix_power( const void * rtdata, Power_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_mix.request_fuel_mix().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359BA94007D:request_level_power'
INLINE_METHODS void FuelController_Actor::transition10_request_level_power( const void * rtdata, Power_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_level.request_fuel_level().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Process_Power_Consume_Request:J4359BAB8034C:consume_to_power'
INLINE_METHODS void FuelController_Actor::transition11_consume_to_power( const int * rtdata, Consume_Protocol::Base * rtport )
{
	// {{{USR
	to_power.consume_value((int)*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Process_Power_Mix_Request:J4359BAD6039A:mix_to_power'
INLINE_METHODS void FuelController_Actor::transition12_mix_to_power( const bool * rtdata, Mix_Protocol::Base * rtport )
{
	// {{{USR
	to_power.mix_value((int)*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Process_Power_Level_Request:J4359BAE70290:level_to_power'
INLINE_METHODS void FuelController_Actor::transition13_level_to_power( const int * rtdata, Level_Protocol::Base * rtport )
{
	// {{{USR
	to_power.level_value((int)*rtdata).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359BDF30010:unload'
INLINE_METHODS void FuelController_Actor::transition14_unload( const void * rtdata, Bus_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	timer.informIn(RTTimespec(10,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Process_Unload:J4359BE4302D0:unload'
INLINE_METHODS void FuelController_Actor::transition15_unload( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	to_level.unload().send();
	to_bus.unload_complete().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359C0010169:decrease_flow'
INLINE_METHODS void FuelController_Actor::transition16_decrease_flow( const void * rtdata, Power_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_consume.decrease_consume().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359C0820244:increase_flow'
INLINE_METHODS void FuelController_Actor::transition17_increase_flow( const void * rtdata, Power_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_consume.increase_consume().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359C68E00AB:engine_on'
INLINE_METHODS void FuelController_Actor::transition18_engine_on( const void * rtdata, Power_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_level.engine_on().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J4359C6CF0273:engine_off'
INLINE_METHODS void FuelController_Actor::transition19_engine_off( const void * rtdata, Power_Comb_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_level.engine_off().send();
	// }}}USR
}
// }}}RME

INLINE_CHAINS void FuelController_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain2_request_consume( void )
{
	// transition ':TOP:Active:J4359B2AE0214:request_consume'
	rtgChainBegin( 2, "request_consume" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_request_consume( msg->data, (Bus_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void FuelController_Actor::chain6_request_mix( void )
{
	// transition ':TOP:Active:J4359B780037B:request_mix'
	rtgChainBegin( 2, "request_mix" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_request_mix( msg->data, (Bus_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void FuelController_Actor::chain3_request_bus_level( void )
{
	// transition ':TOP:Active:J4359B5000197:request_bus_level'
	rtgChainBegin( 2, "request_bus_level" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_request_bus_level( msg->data, (Bus_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void FuelController_Actor::chain14_unload( void )
{
	// transition ':TOP:Active:J4359BDF30010:unload'
	rtgChainBegin( 2, "unload" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition14_unload( msg->data, (Bus_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 9 );
}

INLINE_CHAINS void FuelController_Actor::chain10_request_level_power( void )
{
	// transition ':TOP:Active:J4359BA94007D:request_level_power'
	rtgChainBegin( 2, "request_level_power" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition10_request_level_power( msg->data, (Power_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 6 );
}

INLINE_CHAINS void FuelController_Actor::chain8_request_consume_power( void )
{
	// transition ':TOP:Active:J4359BA6001A6:request_consume_power'
	rtgChainBegin( 2, "request_consume_power" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition8_request_consume_power( msg->data, (Power_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 8 );
}

INLINE_CHAINS void FuelController_Actor::chain9_request_mix_power( void )
{
	// transition ':TOP:Active:J4359BA7D0158:request_mix_power'
	rtgChainBegin( 2, "request_mix_power" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition9_request_mix_power( msg->data, (Power_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 7 );
}

INLINE_CHAINS void FuelController_Actor::chain17_increase_flow( void )
{
	// transition ':TOP:Active:J4359C0820244:increase_flow'
	rtgChainBegin( 2, "increase_flow" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition17_increase_flow( msg->data, (Power_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain16_decrease_flow( void )
{
	// transition ':TOP:Active:J4359C0010169:decrease_flow'
	rtgChainBegin( 2, "decrease_flow" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition16_decrease_flow( msg->data, (Power_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain18_engine_on( void )
{
	// transition ':TOP:Active:J4359C68E00AB:engine_on'
	rtgChainBegin( 2, "engine_on" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition18_engine_on( msg->data, (Power_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain19_engine_off( void )
{
	// transition ':TOP:Active:J4359C6CF0273:engine_off'
	rtgChainBegin( 2, "engine_off" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition19_engine_off( msg->data, (Power_Comb_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain4_consume_to_bus( void )
{
	// transition ':TOP:Process_Bus_Consume_Request:J4359B6130158:consume_to_bus'
	rtgChainBegin( 3, "consume_to_bus" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_consume_to_bus( (const int *)msg->data, (Consume_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain5_level_to_bus( void )
{
	// transition ':TOP:Process_Bus_Level_Request:J4359B64E0223:level_to_bus'
	rtgChainBegin( 4, "level_to_bus" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_level_to_bus( (const int *)msg->data, (Level_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain7_mix_to_bus( void )
{
	// transition ':TOP:Process_Bus_Mix_Request:J4359B7D202DF:mix_to_bus'
	rtgChainBegin( 5, "mix_to_bus" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition7_mix_to_bus( (const bool *)msg->data, (Mix_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain13_level_to_power( void )
{
	// transition ':TOP:Process_Power_Level_Request:J4359BAE70290:level_to_power'
	rtgChainBegin( 6, "level_to_power" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition13_level_to_power( (const int *)msg->data, (Level_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain12_mix_to_power( void )
{
	// transition ':TOP:Process_Power_Mix_Request:J4359BAD6039A:mix_to_power'
	rtgChainBegin( 7, "mix_to_power" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition12_mix_to_power( (const bool *)msg->data, (Mix_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain11_consume_to_power( void )
{
	// transition ':TOP:Process_Power_Consume_Request:J4359BAB8034C:consume_to_power'
	rtgChainBegin( 8, "consume_to_power" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition11_consume_to_power( (const int *)msg->data, (Consume_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void FuelController_Actor::chain15_unload( void )
{
	// transition ':TOP:Process_Unload:J4359BE4302D0:unload'
	rtgChainBegin( 9, "unload" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition15_unload( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 9 );
}

void FuelController_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Active'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_bus'
				switch( signalIndex )
				{
				case Bus_Comb_Protocol::Conjugate::rti_request_fuel_consume:
					chain2_request_consume();
					return;
				case Bus_Comb_Protocol::Conjugate::rti_request_fuel_mix:
					chain6_request_mix();
					return;
				case Bus_Comb_Protocol::Conjugate::rti_request_fuel_level:
					chain3_request_bus_level();
					return;
				case Bus_Comb_Protocol::Conjugate::rti_do_unload:
					chain14_unload();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 2:
				// {{{RME port 'to_power'
				switch( signalIndex )
				{
				case Power_Comb_Protocol::Conjugate::rti_request_fuel_level:
					chain10_request_level_power();
					return;
				case Power_Comb_Protocol::Conjugate::rti_request_fuel_consume:
					chain8_request_consume_power();
					return;
				case Power_Comb_Protocol::Conjugate::rti_request_fuel_mix:
					chain9_request_mix_power();
					return;
				case Power_Comb_Protocol::Conjugate::rti_increase_flow:
					chain17_increase_flow();
					return;
				case Power_Comb_Protocol::Conjugate::rti_decrease_flow:
					chain16_decrease_flow();
					return;
				case Power_Comb_Protocol::Conjugate::rti_engine_on:
					chain18_engine_on();
					return;
				case Power_Comb_Protocol::Conjugate::rti_engine_off:
					chain19_engine_off();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:Process_Bus_Consume_Request'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 3:
				// {{{RME port 'to_consume'
				switch( signalIndex )
				{
				case Consume_Protocol::Base::rti_consume_value:
					chain4_consume_to_bus();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:Process_Bus_Level_Request'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 5:
				// {{{RME port 'to_level'
				switch( signalIndex )
				{
				case Level_Protocol::Base::rti_level_value:
					chain5_level_to_bus();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 5:
			// {{{RME state ':TOP:Process_Bus_Mix_Request'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 4:
				// {{{RME port 'to_mix'
				switch( signalIndex )
				{
				case Mix_Protocol::Base::rti_fuel_mix_value:
					chain7_mix_to_bus();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 6:
			// {{{RME state ':TOP:Process_Power_Level_Request'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 5:
				// {{{RME port 'to_level'
				switch( signalIndex )
				{
				case Level_Protocol::Base::rti_level_value:
					chain13_level_to_power();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 7:
			// {{{RME state ':TOP:Process_Power_Mix_Request'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 4:
				// {{{RME port 'to_mix'
				switch( signalIndex )
				{
				case Mix_Protocol::Base::rti_fuel_mix_value:
					chain12_mix_to_power();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 8:
			// {{{RME state ':TOP:Process_Power_Consume_Request'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 3:
				// {{{RME port 'to_consume'
				switch( signalIndex )
				{
				case Consume_Protocol::Base::rti_consume_value:
					chain11_consume_to_power();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 9:
			// {{{RME state ':TOP:Process_Unload'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 6:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain15_unload();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * FuelController_Actor::getActorData( void ) const
{
	return &FuelController_Actor::rtg_class;
}

const RTActor_class FuelController_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 9
  , FuelController_Actor::rtg_parent_state
  , &FuelController
  , 0
  , (const RTComponentDescriptor *)0
  , 6
  , FuelController_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId FuelController_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
  , 1
  , 1
  , 1
  , 1
  , 1
  , 1
};

const RTPortDescriptor FuelController_Actor::rtg_ports[] =
{
	{
		"to_bus"
	  , (const char *)0
	  , &Bus_Comb_Protocol::Conjugate::rt_class
	  , RTOffsetOf( FuelController_Actor, FuelController_Actor::to_bus )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_power"
	  , (const char *)0
	  , &Power_Comb_Protocol::Conjugate::rt_class
	  , RTOffsetOf( FuelController_Actor, FuelController_Actor::to_power )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_consume"
	  , (const char *)0
	  , &Consume_Protocol::Base::rt_class
	  , RTOffsetOf( FuelController_Actor, FuelController_Actor::to_consume )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_mix"
	  , (const char *)0
	  , &Mix_Protocol::Base::rt_class
	  , RTOffsetOf( FuelController_Actor, FuelController_Actor::to_mix )
	  , 1 // cardinality
	  , 4
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_level"
	  , (const char *)0
	  , &Level_Protocol::Base::rt_class
	  , RTOffsetOf( FuelController_Actor, FuelController_Actor::to_level )
	  , 1 // cardinality
	  , 5
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( FuelController_Actor, FuelController_Actor::timer )
	  , 1 // cardinality
	  , 6
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
