// {{{RME classifier 'Logical View::VANT::VCTRL::VCOMB::Mix'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "Mix.h"
#endif

#include <RTSystem/VANT.h>
#include <Mix.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_controller"
	  , &Mix_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_Mix_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new Mix_Actor( _rts, _ref );
}

const RTActorClass Mix =
{
	(const RTActorClass *)0
  , "Mix"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_Mix_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Active"
};

#define SUPER RTActor

Mix_Actor::Mix_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, mix_status( 0 )
{
}

Mix_Actor::~Mix_Actor( void )
{
}

int Mix_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_controller
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_controller;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Active:J4359970C01E6:request_mix'
INLINE_METHODS void Mix_Actor::transition2_request_mix( const void * rtdata, Mix_Protocol::Conjugate * rtport )
{
	// {{{USR
	to_controller.fuel_mix_value(mix_status).send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J435997BF02F0:active_mix'
INLINE_METHODS void Mix_Actor::transition3_active_mix( const void * rtdata, Mix_Protocol::Conjugate * rtport )
{
	// {{{USR
	mix_status=true;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Active:J435997ED0188:deactive_mix'
INLINE_METHODS void Mix_Actor::transition4_deactive_mix( const void * rtdata, Mix_Protocol::Conjugate * rtport )
{
	// {{{USR
	mix_status=false;
	// }}}USR
}
// }}}RME

INLINE_CHAINS void Mix_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Mix_Actor::chain2_request_mix( void )
{
	// transition ':TOP:Active:J4359970C01E6:request_mix'
	rtgChainBegin( 2, "request_mix" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_request_mix( msg->data, (Mix_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Mix_Actor::chain3_active_mix( void )
{
	// transition ':TOP:Active:J435997BF02F0:active_mix'
	rtgChainBegin( 2, "active_mix" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_active_mix( msg->data, (Mix_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void Mix_Actor::chain4_deactive_mix( void )
{
	// transition ':TOP:Active:J435997ED0188:deactive_mix'
	rtgChainBegin( 2, "deactive_mix" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_deactive_mix( msg->data, (Mix_Protocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void Mix_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Active'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_controller'
				switch( signalIndex )
				{
				case Mix_Protocol::Conjugate::rti_request_fuel_mix:
					chain2_request_mix();
					return;
				case Mix_Protocol::Conjugate::rti_active_mix:
					chain3_active_mix();
					return;
				case Mix_Protocol::Conjugate::rti_deactive_mix:
					chain4_deactive_mix();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * Mix_Actor::getActorData( void ) const
{
	return &Mix_Actor::rtg_class;
}

const RTActor_class Mix_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 2
  , Mix_Actor::rtg_parent_state
  , &Mix
  , 0
  , (const RTComponentDescriptor *)0
  , 1
  , Mix_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 1
  , Mix_Actor::rtg_Mix_fields
};

const RTStateId Mix_Actor::rtg_parent_state[] =
{
	0
  , 1
};

const RTPortDescriptor Mix_Actor::rtg_ports[] =
{
	{
		"to_controller"
	  , (const char *)0
	  , &Mix_Protocol::Conjugate::rt_class
	  , RTOffsetOf( Mix_Actor, Mix_Actor::to_controller )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

const RTFieldDescriptor Mix_Actor::rtg_Mix_fields[] =
{
	// {{{RME classAttribute 'mix_status'
	{
		"mix_status"
	  , RTOffsetOf( Mix_Actor, mix_status )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_bool
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
