// {{{RME classifier 'Logical View::VANT::VCNS::VNAV::NAV_Controller'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "NAV_Controller.h"
#endif

#include <RTSystem/VANT.h>
#include <NAV_Controller.h>
#include <MissionVant.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_bus"
	  , &NAV_PROTOCOL::Conjugate::rt_class
	  , 1 // cardinality
	}
  , {
		"from_controller"
	  , &Navigation_Plan_Protocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_NAV_Controller_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new NAV_Controller_Actor( _rts, _ref );
}

const RTActorClass NAV_Controller =
{
	(const RTActorClass *)0
  , "NAV_Controller"
  , (RTVersionId)0
  , 2
  , rtg_relays
  , new_NAV_Controller_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Waiting"
  , "Trajectory_Plan"
};

#define SUPER RTActor

NAV_Controller_Actor::NAV_Controller_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

NAV_Controller_Actor::~NAV_Controller_Actor( void )
{
}

int NAV_Controller_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_bus
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_bus;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// from_controller
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &from_controller;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Waiting:J436F5D1501B7:traj_data'
INLINE_METHODS void NAV_Controller_Actor::transition2_traj_data( const int * rtdata, NAV_PROTOCOL::Conjugate * rtport )
{
	// {{{USR
	from_controller.request_traj_plan(*rtdata).send();

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:Trajectory_Plan:J436F5D23006E:traj_planned'
INLINE_METHODS void NAV_Controller_Actor::transition3_traj_planned( const int * rtdata, Navigation_Plan_Protocol::Base * rtport )
{
	// {{{USR
	to_bus.traj_planned(*rtdata).send();
	// }}}USR
}
// }}}RME

INLINE_CHAINS void NAV_Controller_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void NAV_Controller_Actor::chain2_traj_data( void )
{
	// transition ':TOP:Waiting:J436F5D1501B7:traj_data'
	rtgChainBegin( 2, "traj_data" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_traj_data( (const int *)msg->data, (NAV_PROTOCOL::Conjugate *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void NAV_Controller_Actor::chain3_traj_planned( void )
{
	// transition ':TOP:Trajectory_Plan:J436F5D23006E:traj_planned'
	rtgChainBegin( 3, "traj_planned" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_traj_planned( (const int *)msg->data, (Navigation_Plan_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void NAV_Controller_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Waiting'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'to_bus'
				switch( signalIndex )
				{
				case NAV_PROTOCOL::Conjugate::rti_traj_plan_request:
					chain2_traj_data();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:Trajectory_Plan'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'from_controller'
				switch( signalIndex )
				{
				case Navigation_Plan_Protocol::Base::rti_traj_planned:
					chain3_traj_planned();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * NAV_Controller_Actor::getActorData( void ) const
{
	return &NAV_Controller_Actor::rtg_class;
}

const RTActor_class NAV_Controller_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 3
  , NAV_Controller_Actor::rtg_parent_state
  , &NAV_Controller
  , 0
  , (const RTComponentDescriptor *)0
  , 2
  , NAV_Controller_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId NAV_Controller_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
};

const RTPortDescriptor NAV_Controller_Actor::rtg_ports[] =
{
	{
		"to_bus"
	  , (const char *)0
	  , &NAV_PROTOCOL::Conjugate::rt_class
	  , RTOffsetOf( NAV_Controller_Actor, NAV_Controller_Actor::to_bus )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"from_controller"
	  , (const char *)0
	  , &Navigation_Plan_Protocol::Base::rt_class
	  , RTOffsetOf( NAV_Controller_Actor, NAV_Controller_Actor::from_controller )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
