// {{{RME classifier 'Logical View::VANT::VCTRL::VPOTE::PowerControl'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "PowerControl.h"
#endif

#include <RTSystem/VANT.h>
#include <PowerControl.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"to_bus"
	  , &Bus_Power_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_flygthcontrol"
	  , &Power_Cont_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_epr"
	  , &Epr_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_egt"
	  , &Egt_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_rpm"
	  , &Rpm_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_fuelcontrol"
	  , &Power_Comb_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"to_manete"
	  , &Manete_Protocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_PowerControl_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new PowerControl_Actor( _rts, _ref );
}

const RTActorClass PowerControl =
{
	(const RTActorClass *)0
  , "PowerControl"
  , (RTVersionId)0
  , 7
  , rtg_relays
  , new_PowerControl_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "IDLE"
  , "START_POWER"
  , "TAKING_OFF_POWER"
  , "NORMAL_FLIGHT_POWER"
  , "LANDING_POWER"
};

#define SUPER RTActor

PowerControl_Actor::PowerControl_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, valuepower( 0 )
{
}

PowerControl_Actor::~PowerControl_Actor( void )
{
}

int PowerControl_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// to_bus
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_bus;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// to_flygthcontrol
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_flygthcontrol;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 2:
		// to_epr
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_epr;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 3:
		// to_egt
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_egt;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 4:
		// to_rpm
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_rpm;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 5:
		// to_fuelcontrol
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_fuelcontrol;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 6:
		// to_manete
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &to_manete;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME enter ':TOP:IDLE'
INLINE_METHODS void PowerControl_Actor::enter2_IDLE( void )
{
	// {{{USR
	to_epr.do_straight().send();
	to_rpm.do_straight().send();
	to_egt.do_straight().send();
	to_manete.do_straight().send();
	to_bus.value_power(valuepower).send();
	to_fuelcontrol.engine_off().send();

	// }}}USR
}
// }}}RME

void PowerControl_Actor::enterStateV( void )
{
	switch( getCurrentState() )
	{
	case 2:
		enter2_IDLE();
		break;
	case 3:
		enter3_START_POWER();
		break;
	case 4:
		enter4_TAKING_OFF_POWER();
		break;
	case 6:
		enter6_LANDING_POWER();
		break;
	default:
		RTActor::enterStateV();
		break;
	}
}

// {{{RME enter ':TOP:START_POWER'
INLINE_METHODS void PowerControl_Actor::enter3_START_POWER( void )
{
	// {{{USR
	to_fuelcontrol.request_fuel_level().send();
	to_fuelcontrol.request_fuel_mix().send();
	to_fuelcontrol.increase_flow().send();

	// }}}USR
}
// }}}RME

// {{{RME enter ':TOP:TAKING_OFF_POWER'
INLINE_METHODS void PowerControl_Actor::enter4_TAKING_OFF_POWER( void )
{
	// {{{USR
	to_fuelcontrol.request_fuel_level().send();
	to_fuelcontrol.request_fuel_mix().send();
	to_manete.request_manete_status().send();
	to_fuelcontrol.increase_flow().send();
	to_fuelcontrol.increase_flow().send();
	// }}}USR
}
// }}}RME

// {{{RME enter ':TOP:LANDING_POWER'
INLINE_METHODS void PowerControl_Actor::enter6_LANDING_POWER( void )
{
	// {{{USR
	to_manete.request_manete_status().send();
	to_fuelcontrol.decrease_flow().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:IDLE:J4359EDC60139:to_start'
INLINE_METHODS void PowerControl_Actor::transition2_to_start( const void * rtdata, Power_Cont_Protocol::Base * rtport )
{
	// {{{USR
	to_fuelcontrol.engine_on().send();
	//to_flygthcontrol.engine_on().send(); RETIRADO 19NOV05
	to_fuelcontrol.increase_flow().send();
	timer1.informIn(RTTimespec(10,0));

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:START_POWER:J4359F592007E:to_taking_off'
INLINE_METHODS void PowerControl_Actor::transition3_to_taking_off( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	to_manete.do_straight().send();
	to_manete.do_manete_up().send();
	timer1.informIn(RTTimespec(50,0));
	to_fuelcontrol.increase_flow().send();

	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:TAKING_OFF_POWER:J4359F64A02C0:to_normal'
INLINE_METHODS void PowerControl_Actor::transition4_to_normal( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	to_manete.do_straight().send();
	timer1.informIn(RTTimespec(180,0));
	timer2.informIn(RTTimespec(20,0));
	to_fuelcontrol.decrease_flow().send();
	to_fuelcontrol.decrease_flow().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:NORMAL_FLIGHT_POWER:J4359F7890224:timer_power'
INLINE_METHODS void PowerControl_Actor::transition5_timer_power( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	to_bus.value_power((int)rtdata).send();
	to_bus.manete_status((int)rtdata).send();
	timer2.informIn(RTTimespec(20,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:NORMAL_FLIGHT_POWER:J4359F81E00EC:powerhigh'
INLINE_METHODS void PowerControl_Actor::transition6_powerhigh( const void * rtdata, Power_Cont_Protocol::Base * rtport )
{
	// {{{USR
	valuepower = 250;
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:NORMAL_FLIGHT_POWER:J4359FA2F02CE:to_landing'
INLINE_METHODS void PowerControl_Actor::transition7_to_landing( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	to_flygthcontrol.low_power().send();
	to_manete.do_manete_down().send();
	timer1.informIn(RTTimespec(60,0));
	to_fuelcontrol.decrease_flow().send();
	to_fuelcontrol.decrease_flow().send();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:IDLE:J438324C402E0:t1'
INLINE_METHODS void PowerControl_Actor::transition9_t1( const void * rtdata, Power_Cont_Protocol::Base * rtport )
{
	// {{{USR
	// denis 22nov05
	//valuepower = (int) *rtdata;
	valuepower = 0;
	printf("MSG_VPOTE: Enginne's off\n\n");
	// }}}USR
}
// }}}RME

INLINE_CHAINS void PowerControl_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void PowerControl_Actor::chain9_t1( void )
{
	// transition ':TOP:IDLE:J438324C402E0:t1'
	rtgChainBegin( 2, "t1" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition9_t1( msg->data, (Power_Cont_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void PowerControl_Actor::chain2_to_start( void )
{
	// transition ':TOP:IDLE:J4359EDC60139:to_start'
	rtgChainBegin( 2, "to_start" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_to_start( msg->data, (Power_Cont_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void PowerControl_Actor::chain3_to_taking_off( void )
{
	// transition ':TOP:START_POWER:J4359F592007E:to_taking_off'
	rtgChainBegin( 3, "to_taking_off" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_to_taking_off( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void PowerControl_Actor::chain4_to_normal( void )
{
	// transition ':TOP:TAKING_OFF_POWER:J4359F64A02C0:to_normal'
	rtgChainBegin( 4, "to_normal" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_to_normal( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void PowerControl_Actor::chain6_powerhigh( void )
{
	// transition ':TOP:NORMAL_FLIGHT_POWER:J4359F81E00EC:powerhigh'
	rtgChainBegin( 5, "powerhigh" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_powerhigh( msg->data, (Power_Cont_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void PowerControl_Actor::chain7_to_landing( void )
{
	// transition ':TOP:NORMAL_FLIGHT_POWER:J4359FA2F02CE:to_landing'
	rtgChainBegin( 5, "to_landing" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition7_to_landing( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 6 );
}

INLINE_CHAINS void PowerControl_Actor::chain5_timer_power( void )
{
	// transition ':TOP:NORMAL_FLIGHT_POWER:J4359F7890224:timer_power'
	rtgChainBegin( 5, "timer_power" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_timer_power( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void PowerControl_Actor::chain8_t1( void )
{
	// transition ':TOP:LANDING_POWER:J4359FB380001:t1'
	rtgChainBegin( 6, "t1" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

void PowerControl_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:IDLE'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'to_flygthcontrol'
				switch( signalIndex )
				{
				case Power_Cont_Protocol::Base::rti_do_engine_off:
					chain9_t1();
					return;
				case Power_Cont_Protocol::Base::rti_do_engine_on:
					chain2_to_start();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:START_POWER'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 6:
				// {{{RME port 'timer1'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain3_to_taking_off();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:TAKING_OFF_POWER'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 6:
				// {{{RME port 'timer1'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain4_to_normal();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 5:
			// {{{RME state ':TOP:NORMAL_FLIGHT_POWER'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'to_flygthcontrol'
				switch( signalIndex )
				{
				case Power_Cont_Protocol::Base::rti_increase_power:
					chain6_powerhigh();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 6:
				// {{{RME port 'timer1'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain7_to_landing();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 7:
				// {{{RME port 'timer2'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain5_timer_power();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 6:
			// {{{RME state ':TOP:LANDING_POWER'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 6:
				// {{{RME port 'timer1'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain8_t1();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * PowerControl_Actor::getActorData( void ) const
{
	return &PowerControl_Actor::rtg_class;
}

const RTActor_class PowerControl_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 6
  , PowerControl_Actor::rtg_parent_state
  , &PowerControl
  , 0
  , (const RTComponentDescriptor *)0
  , 9
  , PowerControl_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 1
  , PowerControl_Actor::rtg_PowerControl_fields
};

const RTStateId PowerControl_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
  , 1
  , 1
  , 1
};

const RTPortDescriptor PowerControl_Actor::rtg_ports[] =
{
	{
		"to_bus"
	  , (const char *)0
	  , &Bus_Power_Protocol::Base::rt_class
	  , RTOffsetOf( PowerControl_Actor, PowerControl_Actor::to_bus )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_flygthcontrol"
	  , (const char *)0
	  , &Power_Cont_Protocol::Base::rt_class
	  , RTOffsetOf( PowerControl_Actor, PowerControl_Actor::to_flygthcontrol )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_epr"
	  , (const char *)0
	  , &Epr_Protocol::Base::rt_class
	  , RTOffsetOf( PowerControl_Actor, PowerControl_Actor::to_epr )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_egt"
	  , (const char *)0
	  , &Egt_Protocol::Base::rt_class
	  , RTOffsetOf( PowerControl_Actor, PowerControl_Actor::to_egt )
	  , 1 // cardinality
	  , 4
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_rpm"
	  , (const char *)0
	  , &Rpm_Protocol::Base::rt_class
	  , RTOffsetOf( PowerControl_Actor, PowerControl_Actor::to_rpm )
	  , 1 // cardinality
	  , 5
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"timer1"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( PowerControl_Actor, PowerControl_Actor::timer1 )
	  , 1 // cardinality
	  , 6
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"timer2"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( PowerControl_Actor, PowerControl_Actor::timer2 )
	  , 1 // cardinality
	  , 7
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"to_fuelcontrol"
	  , (const char *)0
	  , &Power_Comb_Protocol::Base::rt_class
	  , RTOffsetOf( PowerControl_Actor, PowerControl_Actor::to_fuelcontrol )
	  , 1 // cardinality
	  , 8
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"to_manete"
	  , (const char *)0
	  , &Manete_Protocol::Base::rt_class
	  , RTOffsetOf( PowerControl_Actor, PowerControl_Actor::to_manete )
	  , 1 // cardinality
	  , 9
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

const RTFieldDescriptor PowerControl_Actor::rtg_PowerControl_fields[] =
{
	// {{{RME classAttribute 'valuepower'
	{
		"valuepower"
	  , RTOffsetOf( PowerControl_Actor, valuepower )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
