// {{{RME classifier 'Logical View::VANT::VSUP::VBAD::VBAD_STATUS'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "VBAD_STATUS.h"
#endif

#include <RTSystem/VANT.h>
#include <VBAD_STATUS.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"status_port"
	  , &VBAD_PInterface::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"status_internal"
	  , &Status_Protocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_VBAD_STATUS_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new VBAD_STATUS_Actor( _rts, _ref );
}

const RTActorClass VBAD_STATUS =
{
	(const RTActorClass *)0
  , "VBAD_STATUS"
  , (RTVersionId)0
  , 2
  , rtg_relays
  , new_VBAD_STATUS_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "idle"
  , "busy"
};

#define SUPER RTActor

VBAD_STATUS_Actor::VBAD_STATUS_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, boolStatus( false )
{
}

VBAD_STATUS_Actor::~VBAD_STATUS_Actor( void )
{
}

// {{{RME operation 'setStatus(bool)'
void VBAD_STATUS_Actor::setStatus( bool parStatus )
{
	// {{{USR
	// checar o status atual
	// false = IDLE;
	// true  = BUSY;

#define IDLE false;
#define BUSY true;

	boolStatus = parStatus;

	if (boolStatus)
		printf("MSG_STA: DataBus STATUS Turned BUSY\n");
	else
		printf("MSG_STA: DataBus STATUS Turned IDLE\n");
		
	// }}}USR
}
// }}}RME

// {{{RME operation 'returnStatus()'
bool VBAD_STATUS_Actor::returnStatus( void )
{
	// {{{USR
	return boolStatus;
	// }}}USR
}
// }}}RME

int VBAD_STATUS_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// status_port
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &status_port;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// status_internal
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &status_internal;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:idle:J434FC91000EB:turnBusy'
INLINE_METHODS void VBAD_STATUS_Actor::transition2_turnBusy( const bool * rtdata, Status_Protocol::Base * rtport )
{
	// {{{USR
	printf("\n\nMSG_STA: Data Bus Status active...\n");

	// chamar o mtodo de mudana de status
	setStatus(*rtdata);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:busy:J434FCA3101E4:turnIdle'
INLINE_METHODS void VBAD_STATUS_Actor::transition3_turnIdle( const bool * rtdata, Status_Protocol::Base * rtport )
{
	// {{{USR
	printf("\n\nMSG_STA: Data Bus Status active...\n");

	// chamar o mtodo de mudana de status
	setStatus(*rtdata);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:idle:J4357CBB5034D:status'
INLINE_METHODS void VBAD_STATUS_Actor::transition4_status( const void * rtdata, Status_Protocol::Base * rtport )
{
	// {{{USR
	returnStatus();
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:busy:J4357CC0A0244:status'
INLINE_METHODS void VBAD_STATUS_Actor::transition5_status( const void * rtdata, Status_Protocol::Base * rtport )
{
	// {{{USR
	returnStatus();
	// }}}USR
}
// }}}RME

INLINE_CHAINS void VBAD_STATUS_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VBAD_STATUS_Actor::chain2_turnBusy( void )
{
	// transition ':TOP:idle:J434FC91000EB:turnBusy'
	rtgChainBegin( 2, "turnBusy" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_turnBusy( (const bool *)msg->data, (Status_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void VBAD_STATUS_Actor::chain4_status( void )
{
	// transition ':TOP:idle:J4357CBB5034D:status'
	rtgChainBegin( 2, "status" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_status( msg->data, (Status_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VBAD_STATUS_Actor::chain3_turnIdle( void )
{
	// transition ':TOP:busy:J434FCA3101E4:turnIdle'
	rtgChainBegin( 3, "turnIdle" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_turnIdle( (const bool *)msg->data, (Status_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VBAD_STATUS_Actor::chain5_status( void )
{
	// transition ':TOP:busy:J4357CC0A0244:status'
	rtgChainBegin( 3, "status" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_status( msg->data, (Status_Protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

void VBAD_STATUS_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:idle'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'status_internal'
				switch( signalIndex )
				{
				case Status_Protocol::Base::rti_turnBusy:
					chain2_turnBusy();
					return;
				case Status_Protocol::Base::rti_request:
					chain4_status();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:busy'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'status_internal'
				switch( signalIndex )
				{
				case Status_Protocol::Base::rti_turnIdle:
					chain3_turnIdle();
					return;
				case Status_Protocol::Base::rti_request:
					chain5_status();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * VBAD_STATUS_Actor::getActorData( void ) const
{
	return &VBAD_STATUS_Actor::rtg_class;
}

const RTActor_class VBAD_STATUS_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 3
  , VBAD_STATUS_Actor::rtg_parent_state
  , &VBAD_STATUS
  , 0
  , (const RTComponentDescriptor *)0
  , 2
  , VBAD_STATUS_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 1
  , VBAD_STATUS_Actor::rtg_VBAD_STATUS_fields
};

const RTStateId VBAD_STATUS_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
};

const RTPortDescriptor VBAD_STATUS_Actor::rtg_ports[] =
{
	{
		"status_port"
	  , (const char *)0
	  , &VBAD_PInterface::Base::rt_class
	  , RTOffsetOf( VBAD_STATUS_Actor, VBAD_STATUS_Actor::status_port )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"status_internal"
	  , (const char *)0
	  , &Status_Protocol::Base::rt_class
	  , RTOffsetOf( VBAD_STATUS_Actor, VBAD_STATUS_Actor::status_internal )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

const RTFieldDescriptor VBAD_STATUS_Actor::rtg_VBAD_STATUS_fields[] =
{
	// {{{RME classAttribute 'boolStatus'
	{
		"boolStatus"
	  , RTOffsetOf( VBAD_STATUS_Actor, boolStatus )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_bool
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
