// {{{RME classifier 'Logical View::VANT::VCTRL::VCONT::VCONT'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "VCONT.h"
#endif

#include <RTSystem/VANT.h>
#include <VCONT.h>
#include <Aileron_Protocol.h>
#include <Bus_Cont_Protocol.h>
#include <Elevator_Protocol.h>
#include <Flap_Protocol.h>
#include <Power_Cont_Protocol.h>
#include <Rudder_Protocol.h>
extern const RTActorClass Aileron;
extern const RTActorClass Elevator;
extern const RTActorClass Flap;
extern const RTActorClass FlightControl;
extern const RTActorClass Rudder;

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"bus_port"
	  , &Bus_Cont_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
  , {
		"power_port"
	  , &Power_Cont_Protocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_VCONT_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new VCONT_Actor( _rts, _ref );
}

const RTActorClass VCONT =
{
	(const RTActorClass *)0
  , "VCONT"
  , (RTVersionId)0
  , 2
  , rtg_relays
  , new_VCONT_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
};

static const RTInterfaceDescriptor rtg_interfaces_flightControl[] =
{
	{
		"to_bus"
	  , 1
	}
  , {
		"to_power"
	  , 1
	}
  , {
		"to_aileron"
	  , 1
	}
  , {
		"to_elevator"
	  , 1
	}
  , {
		"to_flap"
	  , 1
	}
  , {
		"to_rudder"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_flightControl[] =
{
	{
		0
	  , &Bus_Cont_Protocol::Base::rt_class
	}
  , {
		1
	  , &Power_Cont_Protocol::Base::rt_class
	}
  , {
		2
	  , &Aileron_Protocol::Conjugate::rt_class
	}
  , {
		3
	  , &Elevator_Protocol::Conjugate::rt_class
	}
  , {
		4
	  , &Flap_Protocol::Conjugate::rt_class
	}
  , {
		5
	  , &Rudder_Protocol::Conjugate::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_aileron[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_aileron[] =
{
	{
		0
	  , &Aileron_Protocol::Base::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_elevator[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_elevator[] =
{
	{
		0
	  , &Elevator_Protocol::Base::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_flap[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_flap[] =
{
	{
		0
	  , &Flap_Protocol::Base::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_rudder[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_rudder[] =
{
	{
		0
	  , &Rudder_Protocol::Base::rt_class
	}
};

#define SUPER RTActor

VCONT_Actor::VCONT_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

VCONT_Actor::~VCONT_Actor( void )
{
}

int VCONT_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// bus_port
		if( rtg_repIndex < 1 )
		{
			// flightControl/to_bus
			return flightControl._followIn( rtg_end, 0, rtg_repIndex );
		}
		break;
	case 1:
		// power_port
		if( rtg_repIndex < 1 )
		{
			// flightControl/to_power
			return flightControl._followIn( rtg_end, 1, rtg_repIndex );
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

int VCONT_Actor::_followOutV( RTBindingEnd & rtg_end, int rtg_compId, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_compId )
	{
	case 1:
		// flightControl
		switch( rtg_portId )
		{
		case 0:
			// to_bus
			if( rtg_repIndex < 1 )
			{
				// bus_port
				return _followOut( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 1:
			// to_power
			if( rtg_repIndex < 1 )
			{
				// power_port
				return _followOut( rtg_end, 1, rtg_repIndex );
			}
			break;
		case 2:
			// to_aileron
			if( rtg_repIndex < 1 )
			{
				// aileron/to_controller
				return aileron._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 3:
			// to_elevator
			if( rtg_repIndex < 1 )
			{
				// elevator/to_controller
				return elevator._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 4:
			// to_flap
			if( rtg_repIndex < 1 )
			{
				// flap/to_controller
				return flap._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 5:
			// to_rudder
			if( rtg_repIndex < 1 )
			{
				// rudder/to_controller
				return rudder._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 2:
		// aileron
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// flightControl/to_aileron
				return flightControl._followIn( rtg_end, 2, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 3:
		// elevator
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// flightControl/to_elevator
				return flightControl._followIn( rtg_end, 3, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 4:
		// flap
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// flightControl/to_flap
				return flightControl._followIn( rtg_end, 4, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 5:
		// rudder
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// flightControl/to_rudder
				return flightControl._followIn( rtg_end, 5, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	default:
		break;
	}
	return RTActor::_followOutV( rtg_end, rtg_compId, rtg_portId, rtg_repIndex );
}

void VCONT_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * VCONT_Actor::getActorData( void ) const
{
	return &VCONT_Actor::rtg_class;
}

const RTActor_class VCONT_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 1
  , VCONT_Actor::rtg_parent_state
  , &VCONT
  , 5
  , VCONT_Actor::rtg_capsule_roles
  , 0
  , (const RTPortDescriptor *)0
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId VCONT_Actor::rtg_parent_state[] =
{
	0
};

const RTComponentDescriptor VCONT_Actor::rtg_capsule_roles[] =
{
	{
		"flightControl"
	  , &FlightControl
	  , RTOffsetOf( VCONT_Actor, flightControl )
	  , 1
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 6
	  , rtg_interfaces_flightControl
	  , 6
	  , rtg_bindings_flightControl
	}
  , {
		"aileron"
	  , &Aileron
	  , RTOffsetOf( VCONT_Actor, aileron )
	  , 2
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_aileron
	  , 1
	  , rtg_bindings_aileron
	}
  , {
		"elevator"
	  , &Elevator
	  , RTOffsetOf( VCONT_Actor, elevator )
	  , 3
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_elevator
	  , 1
	  , rtg_bindings_elevator
	}
  , {
		"flap"
	  , &Flap
	  , RTOffsetOf( VCONT_Actor, flap )
	  , 4
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_flap
	  , 1
	  , rtg_bindings_flap
	}
  , {
		"rudder"
	  , &Rudder
	  , RTOffsetOf( VCONT_Actor, rudder )
	  , 5
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_rudder
	  , 1
	  , rtg_bindings_rudder
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
