// {{{RME classifier 'Logical View::VANT::VCTRL::VPOTE::VPOTE'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "VPOTE.h"
#endif

#include <RTSystem/VANT.h>
#include <VPOTE.h>
#include <Bus_Power_Protocol.h>
#include <Egt_Protocol.h>
#include <Epr_Protocol.h>
#include <Manete_Protocol.h>
#include <Power_Comb_Protocol.h>
#include <Power_Cont_Protocol.h>
#include <Rpm_Protocol.h>
extern const RTActorClass EgtControl;
extern const RTActorClass EprControl;
extern const RTActorClass ManetePower;
extern const RTActorClass PowerControl;
extern const RTActorClass RpmControl;

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"bus_port"
	  , &Bus_Power_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"flygth_port"
	  , &Power_Cont_Protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"fuelcontrol_port"
	  , &Power_Comb_Protocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_VPOTE_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new VPOTE_Actor( _rts, _ref );
}

const RTActorClass VPOTE =
{
	(const RTActorClass *)0
  , "VPOTE"
  , (RTVersionId)0
  , 3
  , rtg_relays
  , new_VPOTE_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
};

static const RTInterfaceDescriptor rtg_interfaces_powerControl[] =
{
	{
		"to_bus"
	  , 1
	}
  , {
		"to_flygthcontrol"
	  , 1
	}
  , {
		"to_epr"
	  , 1
	}
  , {
		"to_egt"
	  , 1
	}
  , {
		"to_rpm"
	  , 1
	}
  , {
		"to_fuelcontrol"
	  , 1
	}
  , {
		"to_manete"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_powerControl[] =
{
	{
		0
	  , &Bus_Power_Protocol::Conjugate::rt_class
	}
  , {
		1
	  , &Power_Cont_Protocol::Conjugate::rt_class
	}
  , {
		2
	  , &Epr_Protocol::Conjugate::rt_class
	}
  , {
		3
	  , &Egt_Protocol::Conjugate::rt_class
	}
  , {
		4
	  , &Rpm_Protocol::Conjugate::rt_class
	}
  , {
		5
	  , &Power_Comb_Protocol::Conjugate::rt_class
	}
  , {
		6
	  , &Manete_Protocol::Conjugate::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_manete[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_manete[] =
{
	{
		0
	  , &Manete_Protocol::Base::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_epr[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_epr[] =
{
	{
		0
	  , &Epr_Protocol::Base::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_egt[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_egt[] =
{
	{
		0
	  , &Egt_Protocol::Base::rt_class
	}
};

static const RTInterfaceDescriptor rtg_interfaces_rpm[] =
{
	{
		"to_controller"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_rpm[] =
{
	{
		0
	  , &Rpm_Protocol::Base::rt_class
	}
};

#define SUPER RTActor

VPOTE_Actor::VPOTE_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
{
}

VPOTE_Actor::~VPOTE_Actor( void )
{
}

int VPOTE_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// bus_port
		if( rtg_repIndex < 1 )
		{
			// powerControl/to_bus
			return powerControl._followIn( rtg_end, 0, rtg_repIndex );
		}
		break;
	case 1:
		// flygth_port
		if( rtg_repIndex < 1 )
		{
			// powerControl/to_flygthcontrol
			return powerControl._followIn( rtg_end, 1, rtg_repIndex );
		}
		break;
	case 2:
		// fuelcontrol_port
		if( rtg_repIndex < 1 )
		{
			// powerControl/to_fuelcontrol
			return powerControl._followIn( rtg_end, 5, rtg_repIndex );
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

int VPOTE_Actor::_followOutV( RTBindingEnd & rtg_end, int rtg_compId, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_compId )
	{
	case 1:
		// powerControl
		switch( rtg_portId )
		{
		case 0:
			// to_bus
			if( rtg_repIndex < 1 )
			{
				// bus_port
				return _followOut( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 1:
			// to_flygthcontrol
			if( rtg_repIndex < 1 )
			{
				// flygth_port
				return _followOut( rtg_end, 1, rtg_repIndex );
			}
			break;
		case 2:
			// to_epr
			if( rtg_repIndex < 1 )
			{
				// epr/to_controller
				return epr._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 3:
			// to_egt
			if( rtg_repIndex < 1 )
			{
				// egt/to_controller
				return egt._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 4:
			// to_rpm
			if( rtg_repIndex < 1 )
			{
				// rpm/to_controller
				return rpm._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		case 5:
			// to_fuelcontrol
			if( rtg_repIndex < 1 )
			{
				// fuelcontrol_port
				return _followOut( rtg_end, 2, rtg_repIndex );
			}
			break;
		case 6:
			// to_manete
			if( rtg_repIndex < 1 )
			{
				// manete/to_controller
				return manete._followIn( rtg_end, 0, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 2:
		// manete
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// powerControl/to_manete
				return powerControl._followIn( rtg_end, 6, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 3:
		// epr
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// powerControl/to_epr
				return powerControl._followIn( rtg_end, 2, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 4:
		// egt
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// powerControl/to_egt
				return powerControl._followIn( rtg_end, 3, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	case 5:
		// rpm
		switch( rtg_portId )
		{
		case 0:
			// to_controller
			if( rtg_repIndex < 1 )
			{
				// powerControl/to_rpm
				return powerControl._followIn( rtg_end, 4, rtg_repIndex );
			}
			break;
		default:
			break;
		}
	default:
		break;
	}
	return RTActor::_followOutV( rtg_end, rtg_compId, rtg_portId, rtg_repIndex );
}

void VPOTE_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * VPOTE_Actor::getActorData( void ) const
{
	return &VPOTE_Actor::rtg_class;
}

const RTActor_class VPOTE_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 1
  , VPOTE_Actor::rtg_parent_state
  , &VPOTE
  , 5
  , VPOTE_Actor::rtg_capsule_roles
  , 0
  , (const RTPortDescriptor *)0
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 0
  , (const RTFieldDescriptor *)0
};

const RTStateId VPOTE_Actor::rtg_parent_state[] =
{
	0
};

const RTComponentDescriptor VPOTE_Actor::rtg_capsule_roles[] =
{
	{
		"powerControl"
	  , &PowerControl
	  , RTOffsetOf( VPOTE_Actor, powerControl )
	  , 1
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 7
	  , rtg_interfaces_powerControl
	  , 7
	  , rtg_bindings_powerControl
	}
  , {
		"manete"
	  , &ManetePower
	  , RTOffsetOf( VPOTE_Actor, manete )
	  , 2
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_manete
	  , 1
	  , rtg_bindings_manete
	}
  , {
		"epr"
	  , &EprControl
	  , RTOffsetOf( VPOTE_Actor, epr )
	  , 3
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_epr
	  , 1
	  , rtg_bindings_epr
	}
  , {
		"egt"
	  , &EgtControl
	  , RTOffsetOf( VPOTE_Actor, egt )
	  , 4
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_egt
	  , 1
	  , rtg_bindings_egt
	}
  , {
		"rpm"
	  , &RpmControl
	  , RTOffsetOf( VPOTE_Actor, rpm )
	  , 5
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_rpm
	  , 1
	  , rtg_bindings_rpm
	}
};

#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
