// {{{RME classifier 'Logical View::VANT::VSUP::VSEL::VSEL_ARMAZENAMENTO'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "VSEL_ARMAZENAMENTO.h"
#endif

#include <RTSystem/VANT.h>
#include <VSEL_ARMAZENAMENTO.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"vsel_interface"
	  , &vsel_protocol::Base::rt_class
	  , 1 // cardinality
	}
  , {
		"vsel_internal"
	  , &vsel_protocol::Base::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_VSEL_ARMAZENAMENTO_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new VSEL_ARMAZENAMENTO_Actor( _rts, _ref );
}

const RTActorClass VSEL_ARMAZENAMENTO =
{
	(const RTActorClass *)0
  , "VSEL_ARMAZENAMENTO"
  , (RTVersionId)0
  , 2
  , rtg_relays
  , new_VSEL_ARMAZENAMENTO_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "waiting"
  , "verify"
  , "sendAlarm"
  , "lowBattery"
};

#define SUPER RTActor

VSEL_ARMAZENAMENTO_Actor::VSEL_ARMAZENAMENTO_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RTActor( rtg_rts, rtg_ref )
	, status( false )
	, MINCARGA( 90 )
{
}

VSEL_ARMAZENAMENTO_Actor::~VSEL_ARMAZENAMENTO_Actor( void )
{
}

// {{{RME operation 'VerificarCarga(int)'
void VSEL_ARMAZENAMENTO_Actor::VerificarCarga( int parCarga )
{
	// {{{USR
	// verificar Carga
	status = false;

	intCarga = parCarga;

	if (intCarga < MINCARGA){
		//Carga baixa	
		status = true; 
		printf("MSG_VSEL: Battery charge: %d%%\n", intCarga );
	}

	// }}}USR
}
// }}}RME

// {{{RME operation 'EnviarAlarme()'
void VSEL_ARMAZENAMENTO_Actor::EnviarAlarme( void )
{
	// {{{USR
	printf("MSG_SELL: Sending alarm...\n\n");

	// mandar msg para barramento de dados
	vsel_interface.write('x').send();
	// }}}USR
}
// }}}RME

int VSEL_ARMAZENAMENTO_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// vsel_interface
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &vsel_interface;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	case 1:
		// vsel_internal
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &vsel_internal;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RTActor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Initial:Initial'
INLINE_METHODS void VSEL_ARMAZENAMENTO_Actor::transition1_Initial( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	printf("\n\n\n*** POWER SUPPLY SYSTEM ACTIVATED *** \n\n");
	printf("MSG_VSEL: Now listening... \n");
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:waiting:J43539E990197:verifyCharge'
INLINE_METHODS void VSEL_ARMAZENAMENTO_Actor::transition2_verifyCharge( const int * rtdata, vsel_protocol::Base * rtport )
{
	// {{{USR
	printf("\n\nMSG_VSEL: CHARGE INFORMED...\n");
	printf("MSG_VSEL: Verifing...\n");

	// chamar metodo
	VerificarCarga(*rtdata);

	// disparar o timer
	timer.informIn(RTTimespec(1,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:verify:J43539EAE03BF:timeout'
INLINE_METHODS void VSEL_ARMAZENAMENTO_Actor::transition3_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR


	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:lowBattery:True'
INLINE_METHODS void VSEL_ARMAZENAMENTO_Actor::transition4_True( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	printf("MSG_VSEL: Battery Charge LOW: %d%%\n%", intCarga);

	// chamar o metodo
	EnviarAlarme();

	// disparar o timer
	timer.informIn(RTTimespec(1,0));
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:lowBattery:False'
INLINE_METHODS void VSEL_ARMAZENAMENTO_Actor::transition5_False( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	printf("MSG_VSEL: Battery Charge OK: %d%%\n", intCarga);
	// }}}USR
}
// }}}RME

// {{{RME transition ':TOP:sendAlarm:J43539EBF03AB:timeout'
INLINE_METHODS void VSEL_ARMAZENAMENTO_Actor::transition6_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	printf("MSG_VSEL: Battery Charge: %d%%\n", intCarga);
	// }}}USR
}
// }}}RME

INLINE_CHAINS void VSEL_ARMAZENAMENTO_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	transition1_Initial( msg->data, msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VSEL_ARMAZENAMENTO_Actor::chain2_verifyCharge( void )
{
	// transition ':TOP:waiting:J43539E990197:verifyCharge'
	rtgChainBegin( 2, "verifyCharge" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition2_verifyCharge( (const int *)msg->data, (vsel_protocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void VSEL_ARMAZENAMENTO_Actor::chain3_timeout( void )
{
	// transition ':TOP:verify:J43539EAE03BF:timeout'
	rtgChainBegin( 3, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	if( choicePoint1_lowBattery( msg->data, (Timing::Base *)msg->sap() ) )
		chain4_True();
	else
		chain5_False();
}

// {{{RME choicePoint ':TOP:lowBattery'
INLINE_METHODS int VSEL_ARMAZENAMENTO_Actor::choicePoint1_lowBattery( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	return(status);
	// }}}USR
}
// }}}RME

INLINE_CHAINS void VSEL_ARMAZENAMENTO_Actor::chain4_True( void )
{
	// transition ':TOP:lowBattery:True'
	rtgChainBegin( 5, "True" );
	rtgTransitionBegin();
	transition4_True( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void VSEL_ARMAZENAMENTO_Actor::chain5_False( void )
{
	// transition ':TOP:lowBattery:False'
	rtgChainBegin( 5, "False" );
	rtgTransitionBegin();
	transition5_False( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void VSEL_ARMAZENAMENTO_Actor::chain6_timeout( void )
{
	// transition ':TOP:sendAlarm:J43539EBF03AB:timeout'
	rtgChainBegin( 4, "timeout" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition6_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

void VSEL_ARMAZENAMENTO_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:waiting'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME port 'vsel_interface'
				switch( signalIndex )
				{
				case vsel_protocol::Base::rti_informCarga:
					chain2_verifyCharge();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 3:
				// {{{RME port 'vsel_internal'
				switch( signalIndex )
				{
				case vsel_protocol::Base::rti_informCarga:
					chain2_verifyCharge();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:verify'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain3_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:sendAlarm'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 2:
				// {{{RME port 'timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain6_timeout();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * VSEL_ARMAZENAMENTO_Actor::getActorData( void ) const
{
	return &VSEL_ARMAZENAMENTO_Actor::rtg_class;
}

const RTActor_class VSEL_ARMAZENAMENTO_Actor::rtg_class =
{
	(const RTActor_class *)0
  , rtg_state_names
  , 4
  , VSEL_ARMAZENAMENTO_Actor::rtg_parent_state
  , &VSEL_ARMAZENAMENTO
  , 0
  , (const RTComponentDescriptor *)0
  , 3
  , VSEL_ARMAZENAMENTO_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 2
  , VSEL_ARMAZENAMENTO_Actor::rtg_VSEL_ARMAZENAMENTO_fields
};

const RTStateId VSEL_ARMAZENAMENTO_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
  , 1
};

const RTPortDescriptor VSEL_ARMAZENAMENTO_Actor::rtg_ports[] =
{
	{
		"vsel_interface"
	  , (const char *)0
	  , &vsel_protocol::Base::rt_class
	  , RTOffsetOf( VSEL_ARMAZENAMENTO_Actor, VSEL_ARMAZENAMENTO_Actor::vsel_interface )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( VSEL_ARMAZENAMENTO_Actor, VSEL_ARMAZENAMENTO_Actor::timer )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"vsel_internal"
	  , (const char *)0
	  , &vsel_protocol::Base::rt_class
	  , RTOffsetOf( VSEL_ARMAZENAMENTO_Actor, VSEL_ARMAZENAMENTO_Actor::vsel_internal )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
};

const RTFieldDescriptor VSEL_ARMAZENAMENTO_Actor::rtg_VSEL_ARMAZENAMENTO_fields[] =
{
	// {{{RME classAttribute 'status'
	{
		"status"
	  , RTOffsetOf( VSEL_ARMAZENAMENTO_Actor, status )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_bool
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'MINCARGA'
  , {
		"MINCARGA"
	  , RTOffsetOf( VSEL_ARMAZENAMENTO_Actor, MINCARGA )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME classAttribute 'intCarga'
int intCarga( 100 );
// }}}RME

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
